﻿Imports System.Configuration
Imports Wisys.AllSystem
Imports Wisys.Bm

Public Class ucBMBillOfMaterial

#Region "Variables"

    Private c_oWisysConnection As Wisys.AllSystem.ConnectionInfo

    '//BillOfMaterialTables  
    Private c_oBillOfMaterialTables As Wisys.Bm.BillOfMaterialTables

#End Region

#Region "Constructor"

    Public Sub New(ByRef oConnectionInfo As Wisys.AllSystem.ConnectionInfo)

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        c_oWisysConnection = oConnectionInfo

        '//Instantiate the BillOfMaterialTables object 
        '//    Always assign our Wisys ConnectionInfo object using the object's Connection method
        c_oBillOfMaterialTables = New Wisys.Bm.BillOfMaterialTables()
        c_oBillOfMaterialTables.Connection(c_oWisysConnection)

    End Sub

#End Region

#Region "Private Methods"

    ''' <summary>
    ''' Add an item to a Bill of Material. 
    '''     The Sequence No can be manually assigned or if not assigned it will default to the next Sequence No
    '''     All values are assigned through the class properties 
    ''' </summary>
    ''' <remarks></remarks>
    Private Function AddItemToBillOfMaterial(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0

        Try
            '//Pass the Connection
            c_oBillOfMaterialTables.Connection(c_oWisysConnection)

            '//Assign Properties 
            c_oBillOfMaterialTables.BmPrdStr.Item_no = txtItemNo.Text
            c_oBillOfMaterialTables.BmPrdStr.Comp_item_no = txtComponentItemNo.Text
            '//Sequence No can be manually assinged or it will default to the next Sequence No
            If (String.IsNullOrEmpty(txtSequenceNo.Text.Trim)) Then
                c_oBillOfMaterialTables.BmPrdStr.Seq_no = CInt(txtSequenceNo.Text)
            End If
            c_oBillOfMaterialTables.BmPrdStr.Qty_per_par = CDbl(txtQuantityPer.Text)
            c_oBillOfMaterialTables.BmPrdStr.Scrap_qty = CDbl(txtScrapQuantity.Text)
            c_oBillOfMaterialTables.BmPrdStr.Scrap_factor = CDbl(txtShrinkScrapFactor.Text)
            c_oBillOfMaterialTables.BmPrdStr.Loc = txtComponentLocation.Text
            c_oBillOfMaterialTables.BmPrdStr.Mfg_uom = txtStockUOM.Text
            c_oBillOfMaterialTables.BmPrdStr.Attaching_oper_no = CInt(txtAttachOperation.Text)
            c_oBillOfMaterialTables.BmPrdStr.Activity_fg = txtActivityFlag.Text
            c_oBillOfMaterialTables.BmPrdStr.Effectivity_dt = dtEffectiveDate.Value
            c_oBillOfMaterialTables.BmPrdStr.Obsolete_dt = dtObsoleteDate.Value
            c_oBillOfMaterialTables.BmPrdStr.Bulk_issue_fg = ckBulkIssue.Checked
            c_oBillOfMaterialTables.BmPrdStr.Backflush_fg = ckBackflush.Checked

            p_iWisysReturnValue = c_oBillOfMaterialTables.BmPrdStr.Insert(True, sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    ''' <summary>
    ''' Example of reading a single item on the BOM by a sepcific sequence no
    ''' </summary>
    ''' <remarks></remarks>
    Private Function ReadBOMBySequence(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_bRecordFound As Boolean = False

        Try

            '//Pass the Connection
            c_oBillOfMaterialTables.Connection(c_oWisysConnection)

            '//Read the BOM by Parent Item No and Sequence No
            p_iWisysReturnValue = c_oBillOfMaterialTables.BmPrdStr.Read(txtItemNo.Text, CShort(txtSequenceNo.Text), False, p_bRecordFound, sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

            If (p_bRecordFound) Then
                DisplayBOMInformation(c_oBillOfMaterialTables)
            End If

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    ''' <summary>
    ''' Returns the full product strucutre for the parent item into a datatable
    ''' </summary>
    ''' <remarks></remarks>
    Private Function ReturnFullProductStructure(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_bRecordFound As Boolean = False
        Dim p_dtProductStructure As DataTable = Nothing

        Try

            '//Pass the Connection
            c_oBillOfMaterialTables.Connection(c_oWisysConnection)

            '//Read The Complete Product Structure Into A DataTable
            p_iWisysReturnValue = c_oBillOfMaterialTables.BmPrdStr.ReadStructure(txtItemNo.Text, False, p_bRecordFound, p_dtProductStructure, sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    ''' <summary>
    ''' Example of Updating an item in the Imitmidx_sql table
    ''' </summary>
    ''' <remarks></remarks>
    Private Function UpdateItemOnBOM(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_bRecordFound As Boolean = False

        Try
            '//Pass the Connection
            c_oBillOfMaterialTables.Connection(c_oWisysConnection)

            '//Assign Properties 
            c_oBillOfMaterialTables.BmPrdStr.Comp_item_no = txtComponentItemNo.Text
            c_oBillOfMaterialTables.BmPrdStr.Qty_per_par = CDbl(txtQuantityPer.Text)
            c_oBillOfMaterialTables.BmPrdStr.Scrap_qty = CDbl(txtScrapQuantity.Text)
            c_oBillOfMaterialTables.BmPrdStr.Scrap_factor = CDbl(txtShrinkScrapFactor.Text)
            c_oBillOfMaterialTables.BmPrdStr.Loc = txtComponentLocation.Text
            c_oBillOfMaterialTables.BmPrdStr.Mfg_uom = txtStockUOM.Text
            c_oBillOfMaterialTables.BmPrdStr.Attaching_oper_no = CInt(txtAttachOperation.Text)
            c_oBillOfMaterialTables.BmPrdStr.Activity_fg = txtActivityFlag.Text
            c_oBillOfMaterialTables.BmPrdStr.Effectivity_dt = dtEffectiveDate.Value
            c_oBillOfMaterialTables.BmPrdStr.Obsolete_dt = dtObsoleteDate.Value
            c_oBillOfMaterialTables.BmPrdStr.Bulk_issue_fg = ckBulkIssue.Checked
            c_oBillOfMaterialTables.BmPrdStr.Backflush_fg = ckBackflush.Checked

            p_iWisysReturnValue = c_oBillOfMaterialTables.BmPrdStr.Update(txtItemNo.Text, CShort(txtSequenceNo.Text), True, p_bRecordFound, sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    ''' <summary>
    ''' Remove an item from the bill of material
    ''' </summary>
    ''' <remarks></remarks>
    Private Function DeleteItemFromBOM(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_bRecordFound As Boolean = False

        Try
            '//Pass the Connection
            c_oBillOfMaterialTables.Connection(c_oWisysConnection)

            p_iWisysReturnValue = c_oBillOfMaterialTables.BmPrdStr.Delete(True, txtItemNo.Text, txtComponentItemNo.Text, CShort(txtSequenceNo.Text), sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    Private Sub DisplayBOMInformation(oBillOfMaterialTables As Wisys.Bm.BillOfMaterialTables)

        txtItemNo.Text = oBillOfMaterialTables.BmPrdStr.Item_no
        txtComponentItemNo.Text = oBillOfMaterialTables.BmPrdStr.Comp_item_no
        txtSequenceNo.Text = c_oBillOfMaterialTables.BmPrdStr.Seq_no.ToString
        txtQuantityPer.Text = c_oBillOfMaterialTables.BmPrdStr.Qty_per_par.ToString
        txtScrapQuantity.Text = c_oBillOfMaterialTables.BmPrdStr.Scrap_qty.ToString
        txtShrinkScrapFactor.Text = c_oBillOfMaterialTables.BmPrdStr.Scrap_factor.ToString
        txtComponentLocation.Text = c_oBillOfMaterialTables.BmPrdStr.Loc
        txtStockUOM.Text = c_oBillOfMaterialTables.BmPrdStr.Mfg_uom
        txtAttachOperation.Text = c_oBillOfMaterialTables.BmPrdStr.Attaching_oper_no.ToString
        txtActivityFlag.Text = c_oBillOfMaterialTables.BmPrdStr.Activity_fg
        dtEffectiveDate.Value = c_oBillOfMaterialTables.BmPrdStr.Effectivity_dt
        dtObsoleteDate.Value = c_oBillOfMaterialTables.BmPrdStr.Obsolete_dt
        ckBulkIssue.Checked = c_oBillOfMaterialTables.BmPrdStr.Bulk_issue_fg
        ckBackflush.Checked = c_oBillOfMaterialTables.BmPrdStr.Backflush_fg

    End Sub

#End Region

#Region "Events"

    Private Sub btnAddItemToBOM_Click(sender As System.Object, e As System.EventArgs) Handles btnAddItemToBOM.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""
        Try
            '//Start Wisys SQL Transaction
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(True, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Add Item To Bill Of Material
            p_iWisysReturnValue = AddItemToBillOfMaterial(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Add Item to BOM")
            End If

            '//Close Wisys Connection
            Dim p_eTransactionAction As Wisys.AllSystem.TrxEnums.TransactionAction
            p_eTransactionAction = If(p_iWisysReturnValue = SUCCESSFUL, Wisys.AllSystem.TrxEnums.TransactionAction.Commit, Wisys.AllSystem.TrxEnums.TransactionAction.Rollback)
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(p_eTransactionAction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If


        Catch ex As Exception
            MessageBox.Show(ex.Message, "Add Item to BOM")

        End Try

    End Sub

    Private Sub btnUpdateBOM_Click(sender As System.Object, e As System.EventArgs) Handles btnUpdateBOM.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""
        Try
            '//Start Wisys SQL Transaction
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(True, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Update Bill Of Material
            p_iWisysReturnValue = UpdateItemOnBOM(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Update BOM")
            End If

            '//Close Wisys Connection
            Dim p_eTransactionAction As Wisys.AllSystem.TrxEnums.TransactionAction
            p_eTransactionAction = If(p_iWisysReturnValue = SUCCESSFUL, Wisys.AllSystem.TrxEnums.TransactionAction.Commit, Wisys.AllSystem.TrxEnums.TransactionAction.Rollback)
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(p_eTransactionAction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If


        Catch ex As Exception
            MessageBox.Show(ex.Message, "Update BOM")

        End Try


    End Sub

    Private Sub btnDeleteItemFromBOM_Click(sender As System.Object, e As System.EventArgs) Handles btnDeleteItemFromBOM.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""
        Try
            '//Start Wisys SQL Transaction
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(True, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Delete Item from Bill Of Material
            p_iWisysReturnValue = DeleteItemFromBOM(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Delete BOM")
            End If

            '//Close Wisys Connection
            Dim p_eTransactionAction As Wisys.AllSystem.TrxEnums.TransactionAction
            p_eTransactionAction = If(p_iWisysReturnValue = SUCCESSFUL, Wisys.AllSystem.TrxEnums.TransactionAction.Commit, Wisys.AllSystem.TrxEnums.TransactionAction.Rollback)
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(p_eTransactionAction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If


        Catch ex As Exception
            MessageBox.Show(ex.Message, "Delete BOM")

        End Try

    End Sub

    Private Sub btnReadBOMBySequence_Click(sender As System.Object, e As System.EventArgs) Handles btnReadBOMBySequence.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""
        Try
            '//Open Wisys Connection
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(False, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Read Bill Of Material By Sequence
            p_iWisysReturnValue = ReadBOMBySequence(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Read BOM")
            End If

            '//Close Wisys Connection
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(TrxEnums.TransactionAction.NoTransaction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If


        Catch ex As Exception
            MessageBox.Show(ex.Message, "Read BOM")

        End Try


    End Sub

    Private Sub btnReturnFullStructure_Click(sender As System.Object, e As System.EventArgs) Handles btnReturnFullStructure.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""
        Try
            '//Open Wisys Connection
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(False, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Return Full Product Structure
            p_iWisysReturnValue = ReturnFullProductStructure(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Read Product Structure")
            End If

            '//Close Wisys Connection
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(TrxEnums.TransactionAction.NoTransaction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If


        Catch ex As Exception
            MessageBox.Show(ex.Message, "Read Product Structure")

        End Try


    End Sub

#End Region







End Class
