﻿Imports System.Configuration
Imports System.Collections.Generic
Imports System.Reflection
Imports Wisys.AllSystem
Imports Wisys.Oe

''' <summary>
''' Example: 
''' * Create, Read, Save, and Delete an Order in Progression and Macola ES
''' * Modify/Delete a specific Line Item
''' * Access Progression and Macola ES Specific Fields
''' * How to Modify The Bill-To / SHip-To Address fields
''' * Header and Line Comments
''' * Default Price and Cost
''' * Skip Customer Credit Check
''' * Audit Records
''' 
''' </summary>
''' <remarks></remarks>
''' <example></example>
Public Class ucOECreateOrder

#Region "Variables"

    Private c_oWisysConnection As Wisys.AllSystem.ConnectionInfo
    Private c_oWisysTrxCommon As Wisys.AllSystem.TrxCommon
    Private c_oOrderEntryTables As Wisys.Oe.OrderEntryTables
    Private c_oInventoryTables As Wisys.Im.InventoryTables

#End Region

#Region "Constructor"

    Public Sub New(ByRef oConnectionInfo As Wisys.AllSystem.ConnectionInfo)

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        LoadOrderTypes()
        c_oWisysConnection = oConnectionInfo

        '//Instantiate the OrderEntryTables object 
        '//    Assign our Wisys ConnectionInfo object using the object's Connection method
        Dim p_sWisysReturnMessage As String = ""
        c_oOrderEntryTables = New Wisys.Oe.OrderEntryTables()
        c_oOrderEntryTables.Connection(c_oWisysConnection, p_sWisysReturnMessage)

        c_oInventoryTables = New Wisys.Im.InventoryTables()
        c_oInventoryTables.Connection(c_oWisysConnection)

        c_oWisysTrxCommon = New Wisys.AllSystem.TrxCommon()
        c_oWisysTrxCommon.Connection(c_oWisysConnection)

    End Sub

#End Region

#Region "Private Methods"

    Private Sub LoadOrderTypes()

        Dim p_oDictionary As New Dictionary(Of String, String)()
        p_oDictionary.Add(cAppHelper.GetStringValue(cAppHelper.OrderType.Order), cAppHelper.OrderType.Order.ToString)
        p_oDictionary.Add(cAppHelper.GetStringValue(cAppHelper.OrderType.CreditMemo), cAppHelper.OrderType.CreditMemo.ToString)
        p_oDictionary.Add(cAppHelper.GetStringValue(cAppHelper.OrderType.Quote), cAppHelper.OrderType.Quote.ToString)
        cbOrderType.DataSource = New BindingSource(p_oDictionary, Nothing)
        cbOrderType.DisplayMember = "Value"
        cbOrderType.ValueMember = "Key"

    End Sub

    Private Sub InitializeUI()

        cbOrderType.SelectedText = ""
        txtOrderNo.Text = ""
        dtOrderDate.Value = DateTime.Now
        dtShipDate.Value = DateTime.Now
        txtOrderStatus.Text = ""

        txtCustomerNo.Text = ""
        txtShipToName.Text = ""
        ckSkipCreditCheck.Checked = False

        txtSalesPersonNo1.Text = ""
        txtSalesPercentage1.Text = ""
        txtSalesPersonNo2.Text = ""
        txtSalesPercentage2.Text = ""

        txtBillToName.Text = ""
        txtBillToAddress1.Text = ""
        txtBillToAddress2.Text = ""
        txtBillToAddress3.Text = ""
        txtBillToAddress4.Text = ""
        txtBillToCountry.Text = ""

        txtShipToName.Text = ""
        txtShipToAddress1.Text = ""
        txtShipToAddress2.Text = ""
        txtShipToAddress3.Text = ""
        txtShipToAddress4.Text = ""
        txtShipToCountry.Text = ""

        EnableDisableFields(True)

    End Sub

    ''' <summary>
    ''' Create An OE Order Header And Order Lines In A Single SQL Transaction
    ''' </summary>
    ''' <remarks></remarks>
    Private Function CreateOEOrder(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_bItemFound As Boolean = False
        Dim p_eOrderType As Wisys.AllSystem.TrxEnums.OeOrderTypeEnum
        Dim p_eLineItemType As Wisys.AllSystem.TrxEnums.OrderLineType = TrxEnums.OrderLineType.InventoryItem

        Try
            '//Pass the Connection
            c_oOrderEntryTables.Connection(c_oWisysConnection, sWisysReturnMessage)

            '//Get Order Type Enumeration
            p_eOrderType = c_oWisysTrxCommon.ReturnOEOrderTypeEnum(cbOrderType.SelectedValue)

            '//Assign property values

            '//Bill To / Ship To will default to Customer/Alt Address unless assigned directly here
            c_oOrderEntryTables.OrderHeader.Bill_to_name = txtBillToName.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_1 = txtBillToAddress1.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_2 = txtBillToAddress2.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_3 = txtBillToAddress3.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_4 = txtBillToAddress4.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_country = txtBillToCountry.Text

            c_oOrderEntryTables.OrderHeader.Ship_to_name = txtShipToName.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_1 = txtShipToAddress1.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_2 = txtShipToAddress2.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_3 = txtShipToAddress3.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_4 = txtShipToAddress4.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_country = txtShipToCountry.Text

            c_oOrderEntryTables.OrderHeader.Shipped_dt = dtShipDate.Value

            '//Macola Version Specific Properties
            If (c_oWisysConnection.MacolaVersion = TrxEnums.MacolaVersion.VersionES) Then
                c_oOrderEntryTables.OrderHeader.EsFields.Extra_9 = "ES Extra 9"
                c_oOrderEntryTables.OrderHeader.EsFields.Slspsn_no = If(String.IsNullOrWhiteSpace(txtSalesPersonNo1.Text), 0, CInt(txtSalesPersonNo1.Text))
                c_oOrderEntryTables.OrderHeader.EsFields.Slspsn_no_2 = If(String.IsNullOrWhiteSpace(txtSalesPersonNo2.Text), 0, CInt(txtSalesPersonNo2.Text))
            Else
                c_oOrderEntryTables.OrderHeader.ProgressionFields.Slspsn_no = txtSalesPersonNo1.Text
                c_oOrderEntryTables.OrderHeader.ProgressionFields.Slspsn_no_2 = txtSalesPersonNo2.Text
            End If
            c_oOrderEntryTables.OrderHeader.Slspsn_pct_comm = If(String.IsNullOrWhiteSpace(txtSalesPercentage1.Text), 0, CDbl(txtSalesPercentage1.Text))
            c_oOrderEntryTables.OrderHeader.Slspsn_pct_comm_2 = If(String.IsNullOrWhiteSpace(txtSalesPercentage2.Text), 0, CDbl(txtSalesPercentage2.Text))



            c_oOrderEntryTables.OrderHeader.HeaderComment = "This is a comment that extends past Macolas 40 character per comment record" '//Header comments will be parsed in 40 charcter chunks honoring word breaks
            c_oOrderEntryTables.OrderHeader.SkipCreditCheck = ckSkipCreditCheck.Checked '//Determines whether customer credit check should occur 

            '//Insert 
            '//    The insert method parameters contain the minimum required information to create an oeordhdr record.
            '//    Properties not set manually will default from corresponding tables, control files etc... 
            p_iWisysReturnValue = c_oOrderEntryTables.OrderHeader.Insert(p_eOrderType _
                                                                       , txtOrderNo.Text _
                                                                       , txtCustomerNo.Text _
                                                                       , txtAlternateAddress.Text _
                                                                       , dtOrderDate.Value _
                                                                       , sWisysReturnMessage)


            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

            txtOrderNo.Text = c_oOrderEntryTables.OrderHeader.Ord_no

            '//Add OE Line Items

            '//Read OE Order Header

            Dim p_bRecordFound As Boolean = False
            p_iWisysReturnValue = c_oOrderEntryTables.OrderHeader.Read(p_eOrderType _
                                                                     , txtOrderNo.Text _
                                                                     , False _
                                                                     , True _
                                                                     , True _
                                                                     , p_bRecordFound _
                                                                     , sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If
            If (p_bRecordFound = False) Then
                sWisysReturnMessage = Wisys.AllSystem.Constants.ORDER_NO_FOUND
                Return FAILED
            End If

            '//Loop through order line grid to add line items 
            For Each p_oOrderEntryLine As cOrderEntryLine In UcOELineGrid1.GridLineCollection
                If (Not String.IsNullOrEmpty(p_oOrderEntryLine.ItemNo)) Then
                    Dim p_oOrderLine As Wisys.Oe.OEORDLIN
                    Dim p_iRtnLineNo As Integer = 0

                    p_bItemFound = False

                    '//The Wisys.OrderEntryTables.OrderHeader.IsBulkTransaction property can be utilized when adding multiple line items in a single transaction.
                    '//This Property improves performance by updating the Order Header accumulators after all line items have been added
                    '//The ***Wisys.OrderEntryTables.OrderHeader.FinalizeBulkTransaction*** Method must be called after all Line Items have been added
                    c_oOrderEntryTables.OrderHeader.IsBulkTransaction = True


                    '//Determine If Item Is Inventory or Non Inventory
                    c_oInventoryTables.Connection(c_oWisysConnection)
                    p_iWisysReturnValue = c_oInventoryTables.Imitmidx.Read(p_oOrderEntryLine.ItemNo _
                                                                         , p_oOrderEntryLine.Location _
                                                                         , True _
                                                                         , p_bItemFound _
                                                                         , sWisysReturnMessage)
                    If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                        Return p_iWisysReturnValue
                    End If

                    If (p_bItemFound = True) Then
                        p_eLineItemType = TrxEnums.OrderLineType.InventoryItem
                    Else
                        p_eLineItemType = TrxEnums.OrderLineType.NonStockedItem
                    End If

                    '//The OrderLines.Add Method Contains All Required Values To Create An OE Line Item
                    '//Passing -1 (Negative one) to the Price and Cost parameters will use the default Price and Cost
                    p_oOrderLine = c_oOrderEntryTables.OrderHeader.OrderLines.Add(p_eLineItemType _
                                                                                , p_oOrderEntryLine.ItemNo _
                                                                                , p_oOrderEntryLine.Location _
                                                                                , p_oOrderEntryLine.QtyOrdered _
                                                                                , p_oOrderEntryLine.QtyToShip _
                                                                                , p_oOrderEntryLine.Price _
                                                                                , p_oOrderEntryLine.Cost _
                                                                                , p_oOrderEntryLine.RequestDate _
                                                                                , p_oOrderEntryLine.PromiseDate _
                                                                                , p_oOrderEntryLine.RequestShipDate _
                                                                                , False _
                                                                                , sWisysReturnMessage)
                    '//Assign Additional Order Line Properties

                    '//Add Bin, Lot, Serial information for Credit Memos
                    If (p_bItemFound AndAlso _
                        p_eOrderType = TrxEnums.OeOrderTypeEnum.CreditMemo AndAlso _
                        (c_oInventoryTables.Imitmidx.Ser_lot_fg = "L" OrElse _
                         c_oInventoryTables.Imitmidx.Ser_lot_fg = "S" OrElse _
                         c_oInventoryTables.Imitmidx.Iminvloc.Binned)) Then

                        Dim p_oWisysBinCollection As New Wisys.AllSystem.DataStructures.ItemBins
                        Dim p_oWisysSerialLotCollection As New Wisys.AllSystem.DataStructures.ItemSerialLots

                        Dim p_frmBinSerialLotDialog As New frmBinSerialLotDialog()
                        p_frmBinSerialLotDialog.lblTitle.Text = "Item No: " & p_oOrderEntryLine.ItemNo.Trim() & " Qty Returned: " & p_oOrderEntryLine.QtyToShip.ToString()
                        p_frmBinSerialLotDialog.ShowDialog()
                        p_frmBinSerialLotDialog.UcBinSerialLotGrid1.GetBinSerialLotCollection(p_oWisysBinCollection, p_oWisysSerialLotCollection)
                        If (p_oWisysBinCollection IsNot Nothing AndAlso p_oWisysBinCollection.Count > 0) Then
                            p_oOrderLine.ReturnBins = p_oWisysBinCollection
                        End If
                        If (p_oWisysSerialLotCollection IsNot Nothing AndAlso p_oWisysSerialLotCollection.Count > 0) Then
                            p_oOrderLine.ReturnSerialLots = p_oWisysSerialLotCollection
                        End If
                    End If

                    '//User For Audit Records
                    p_oOrderLine.User = Environment.UserName

                    '//Order Line Comments (OELINCMT_SQL)
                    p_oOrderLine.LineNotes = "Here are some line comments. They will span multiple lines if they go beyond the 40 character limit."

                    '//Allow Non Inventory Items If OECTLFIL Set To Warn
                    p_oOrderLine.AllowInactiveIfWarn = True

                    '//Insert Line Item
                    p_iWisysReturnValue = p_oOrderLine.Insert(p_iRtnLineNo, sWisysReturnMessage)
                    If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                        Return p_iWisysReturnValue
                    End If
                End If

            Next

            '//FinalizeBulkTransaction should be used in conjuction with the IsBulkTransaction property
            p_iWisysReturnValue = c_oOrderEntryTables.OrderHeader.FinalizeBulkTransaction(sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    Private Function LoadOrder(sOrderType As String, sOrderNo As String, ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0

        Try

            '//Pass the Connection
            c_oOrderEntryTables.Connection(c_oWisysConnection, sWisysReturnMessage)

            '//Read The Order And Line Items
            Dim p_eOrderType As Wisys.AllSystem.TrxEnums.OeOrderTypeEnum
            Dim p_bRecordFound As Boolean = False
            p_eOrderType = c_oWisysTrxCommon.ReturnOEOrderTypeEnum(sOrderType)
            p_iWisysReturnValue = c_oOrderEntryTables.OrderHeader.Read(p_eOrderType _
                                                                     , sOrderNo _
                                                                     , False _
                                                                     , False _
                                                                     , False _
                                                                     , p_bRecordFound _
                                                                     , sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If
            If (p_bRecordFound = False) Then
                sWisysReturnMessage = Wisys.AllSystem.Constants.ORDER_NO_FOUND
                Return FAILED
            End If

            DisplayOrder()

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    Private Function UpdateOEOrder(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_bItemFound As Boolean = False
        Dim p_eOrderType As Wisys.AllSystem.TrxEnums.OeOrderTypeEnum
        Dim p_eLineItemType As Wisys.AllSystem.TrxEnums.OrderLineType = TrxEnums.OrderLineType.InventoryItem

        Try
            '//Pass the Connection
            c_oOrderEntryTables.Connection(c_oWisysConnection, sWisysReturnMessage)

            '//Get Order Type Enumeration
            p_eOrderType = c_oWisysTrxCommon.ReturnOEOrderTypeEnum(cbOrderType.SelectedValue)

            '//Assign property values

            '//Update Bill To / Ship To Information
            c_oOrderEntryTables.OrderHeader.Bill_to_name = txtBillToName.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_1 = txtBillToAddress1.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_2 = txtBillToAddress2.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_3 = txtBillToAddress3.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_addr_4 = txtBillToAddress4.Text
            c_oOrderEntryTables.OrderHeader.Bill_to_country = txtBillToCountry.Text

            c_oOrderEntryTables.OrderHeader.Ship_to_name = txtShipToName.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_1 = txtShipToAddress1.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_2 = txtShipToAddress2.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_3 = txtShipToAddress3.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_addr_4 = txtShipToAddress4.Text
            c_oOrderEntryTables.OrderHeader.Ship_to_country = txtShipToCountry.Text

            '//Macola Version Specific Properties
            If (c_oWisysConnection.MacolaVersion = TrxEnums.MacolaVersion.VersionES) Then
                c_oOrderEntryTables.OrderHeader.EsFields.Slspsn_no = txtSalesPersonNo1.Text
                c_oOrderEntryTables.OrderHeader.EsFields.Slspsn_no_2 = txtSalesPersonNo2.Text
            Else
                c_oOrderEntryTables.OrderHeader.ProgressionFields.Slspsn_no = txtSalesPersonNo1.Text
                c_oOrderEntryTables.OrderHeader.ProgressionFields.Slspsn_no_2 = txtSalesPersonNo2.Text
            End If
            c_oOrderEntryTables.OrderHeader.Slspsn_pct_comm = If(String.IsNullOrWhiteSpace(txtSalesPercentage1.Text), 0, CDbl(txtSalesPercentage1.Text))
            c_oOrderEntryTables.OrderHeader.Slspsn_pct_comm_2 = If(String.IsNullOrWhiteSpace(txtSalesPercentage2.Text), 0, CDbl(txtSalesPercentage2.Text))
            c_oOrderEntryTables.OrderHeader.SkipCreditCheck = ckSkipCreditCheck.Checked '//Determines whether customer credit check should occur 

            '//Update OEORDHDR
            p_iWisysReturnValue = c_oOrderEntryTables.OrderHeader.Update(sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

            '//Update OE Line Items From Grid
            For Each p_oOrderEntryLine As cOrderEntryLine In UcOELineGrid1.GridLineCollection

                Dim p_oOrderLine As OEORDLIN

                If (p_oOrderEntryLine.LineNo > 0) Then
                    p_oOrderLine = GetOEOrderLine(c_oOrderEntryTables.OrderHeader.OrderLines, p_oOrderEntryLine.LineNo)
                    If (p_oOrderLine Is Nothing) Then
                        sWisysReturnMessage = Wisys.AllSystem.Constants.ORDER_LINE_NOT_FOUND
                        Return FAILED
                    End If
                    p_oOrderLine.Qty_ordered = p_oOrderEntryLine.QtyOrdered
                    If (p_eOrderType = TrxEnums.OeOrderTypeEnum.CreditMemo) Then
                        p_oOrderLine.Qty_return_to_stk = p_oOrderEntryLine.QtyToShip
                    Else
                        p_oOrderLine.Qty_to_ship = p_oOrderEntryLine.QtyToShip
                    End If

                    p_oOrderLine.Unit_cost = p_oOrderEntryLine.Cost
                    p_oOrderLine.Unit_price = p_oOrderEntryLine.Price
                    p_oOrderLine.Request_dt = p_oOrderEntryLine.RequestDate
                    p_oOrderLine.Req_ship_dt = p_oOrderEntryLine.RequestShipDate
                    p_oOrderLine.Promise_dt = p_oOrderEntryLine.PromiseDate

                    '//Add Bin, Lot, Serial information for Credit Memos
                    If (p_eOrderType = TrxEnums.OeOrderTypeEnum.CreditMemo AndAlso _
                        (c_oInventoryTables.Imitmidx.Ser_lot_fg = "L" OrElse _
                         c_oInventoryTables.Imitmidx.Ser_lot_fg = "S" OrElse _
                         c_oInventoryTables.Imitmidx.Iminvloc.Binned)) Then

                        Dim p_oWisysBinCollection As New Wisys.AllSystem.DataStructures.ItemBins
                        Dim p_oWisysSerialLotCollection As New Wisys.AllSystem.DataStructures.ItemSerialLots

                        Dim p_frmBinSerialLotDialog As New frmBinSerialLotDialog()
                        p_frmBinSerialLotDialog.lblTitle.Text = "Item No: " & p_oOrderEntryLine.ItemNo.Trim() & " | Qty Returned: " & p_oOrderEntryLine.QtyToShip.ToString()
                        p_frmBinSerialLotDialog.ShowDialog()
                        p_frmBinSerialLotDialog.UcBinSerialLotGrid1.GetBinSerialLotCollection(p_oWisysBinCollection, p_oWisysSerialLotCollection)
                        If (p_oWisysBinCollection IsNot Nothing AndAlso p_oWisysBinCollection.Count > 0) Then
                            p_oOrderLine.ReturnBins = p_oWisysBinCollection
                        End If
                        If (p_oWisysSerialLotCollection IsNot Nothing AndAlso p_oWisysSerialLotCollection.Count > 0) Then
                            p_oOrderLine.ReturnSerialLots = p_oWisysSerialLotCollection
                        End If
                    End If

                    p_iWisysReturnValue = p_oOrderLine.Update(True, sWisysReturnMessage)
                    If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                        Return p_iWisysReturnValue
                    End If
                Else
                    '//Add A New Line Item To An Existing Order
                    Dim p_iRtnLineNo As Integer = 0

                    '//Determine If Item Is Inventory or Non Inventory
                    c_oInventoryTables.Connection(c_oWisysConnection)
                    p_iWisysReturnValue = c_oInventoryTables.Imitmidx.Read(p_oOrderEntryLine.ItemNo _
                                                                         , p_oOrderEntryLine.Location _
                                                                         , True _
                                                                         , p_bItemFound _
                                                                         , sWisysReturnMessage)
                    If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                        Return p_iWisysReturnValue
                    End If

                    If (p_bItemFound = True) Then
                        p_eLineItemType = TrxEnums.OrderLineType.InventoryItem
                    Else
                        p_eLineItemType = TrxEnums.OrderLineType.NonStockedItem
                    End If

                    p_oOrderLine = c_oOrderEntryTables.OrderHeader.OrderLines.Add(p_eLineItemType _
                                                                               , p_oOrderEntryLine.ItemNo _
                                                                               , p_oOrderEntryLine.Location _
                                                                               , p_oOrderEntryLine.QtyOrdered _
                                                                               , p_oOrderEntryLine.QtyToShip _
                                                                               , p_oOrderEntryLine.Price _
                                                                               , p_oOrderEntryLine.Cost _
                                                                               , p_oOrderEntryLine.RequestDate _
                                                                               , p_oOrderEntryLine.PromiseDate _
                                                                               , p_oOrderEntryLine.RequestShipDate _
                                                                               , False _
                                                                               , sWisysReturnMessage)

                    '//User For Audit Records
                    p_oOrderLine.User = Environment.UserName

                    '//Allow Non Inventory Items If OECTLFIL Set To Warn
                    p_oOrderLine.AllowInactiveIfWarn = True

                    '//Add Bin, Lot, Serial information for Credit Memos
                    If (p_bItemFound AndAlso _
                        p_eOrderType = TrxEnums.OeOrderTypeEnum.CreditMemo AndAlso _
                        (c_oInventoryTables.Imitmidx.Ser_lot_fg = "L" OrElse _
                         c_oInventoryTables.Imitmidx.Ser_lot_fg = "S" OrElse _
                         c_oInventoryTables.Imitmidx.Iminvloc.Binned)) Then

                        Dim p_oWisysBinCollection As New Wisys.AllSystem.DataStructures.ItemBins
                        Dim p_oWisysSerialLotCollection As New Wisys.AllSystem.DataStructures.ItemSerialLots

                        Dim p_frmBinSerialLotDialog As New frmBinSerialLotDialog()
                        p_frmBinSerialLotDialog.lblTitle.Text = "Item No: " & p_oOrderEntryLine.ItemNo.Trim() & " | Qty Returned: " & p_oOrderEntryLine.QtyToShip.ToString()
                        p_frmBinSerialLotDialog.ShowDialog()
                        p_frmBinSerialLotDialog.UcBinSerialLotGrid1.GetBinSerialLotCollection(p_oWisysBinCollection, p_oWisysSerialLotCollection)
                        If (p_oWisysBinCollection IsNot Nothing AndAlso p_oWisysBinCollection.Count > 0) Then
                            p_oOrderLine.ReturnBins = p_oWisysBinCollection
                        End If
                        If (p_oWisysSerialLotCollection IsNot Nothing AndAlso p_oWisysSerialLotCollection.Count > 0) Then
                            p_oOrderLine.ReturnSerialLots = p_oWisysSerialLotCollection
                        End If

                    End If

                    '//Insert A New Line Item
                    p_iWisysReturnValue = p_oOrderLine.Insert(p_iRtnLineNo, sWisysReturnMessage)
                    If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                        Return p_iWisysReturnValue
                    End If

                End If
            Next

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    Private Function GetOEOrderLine(OrderLines As Wisys.Oe.cOrderLines, iLineNo As Integer) As Wisys.Oe.OEORDLIN

        For Each p_oOrderLine As Wisys.Oe.OEORDLIN In OrderLines
            If p_oOrderLine.Line_no = iLineNo Then
                Return p_oOrderLine
            End If
        Next

        Return Nothing

    End Function

    Private Sub DisplayOrder()

        txtOrderNo.Text = c_oOrderEntryTables.OrderHeader.Ord_no
        dtOrderDate.Value = c_oOrderEntryTables.OrderHeader.Ord_dt
        dtShipDate.Value = c_oOrderEntryTables.OrderHeader.Shipping_dt

        txtCustomerNo.Text = c_oOrderEntryTables.OrderHeader.Cus_no
        txtShipToName.Text = c_oOrderEntryTables.OrderHeader.Ship_to_name
        txtOrderStatus.Text = c_oOrderEntryTables.OrderHeader.Status

        If (c_oWisysConnection.MacolaVersion = TrxEnums.MacolaVersion.VersionES) Then
            txtSalesPersonNo1.Text = c_oOrderEntryTables.OrderHeader.EsFields.Slspsn_no
            txtSalesPersonNo2.Text = c_oOrderEntryTables.OrderHeader.EsFields.Slspsn_no_2
        Else
            txtSalesPersonNo1.Text = c_oOrderEntryTables.OrderHeader.ProgressionFields.Slspsn_no
            txtSalesPersonNo2.Text = c_oOrderEntryTables.OrderHeader.ProgressionFields.Slspsn_no_2
        End If

        txtSalesPercentage1.Text = c_oOrderEntryTables.OrderHeader.Slspsn_pct_comm
        txtSalesPercentage2.Text = c_oOrderEntryTables.OrderHeader.Slspsn_pct_comm_2

        txtBillToName.Text = c_oOrderEntryTables.OrderHeader.Bill_to_name
        txtBillToAddress1.Text = c_oOrderEntryTables.OrderHeader.Bill_to_addr_1
        txtBillToAddress2.Text = c_oOrderEntryTables.OrderHeader.Bill_to_addr_2
        txtBillToAddress3.Text = c_oOrderEntryTables.OrderHeader.Bill_to_addr_3
        txtBillToAddress4.Text = c_oOrderEntryTables.OrderHeader.Bill_to_addr_4
        txtBillToCountry.Text = c_oOrderEntryTables.OrderHeader.Bill_to_country

        txtShipToName.Text = c_oOrderEntryTables.OrderHeader.Ship_to_name
        txtShipToAddress1.Text = c_oOrderEntryTables.OrderHeader.Ship_to_addr_1
        txtShipToAddress2.Text = c_oOrderEntryTables.OrderHeader.Ship_to_addr_2
        txtShipToAddress3.Text = c_oOrderEntryTables.OrderHeader.Ship_to_addr_3
        txtShipToAddress4.Text = c_oOrderEntryTables.OrderHeader.Ship_to_addr_4
        txtShipToCountry.Text = c_oOrderEntryTables.OrderHeader.Ship_to_country

        UcOELineGrid1.AddLineItems(c_oOrderEntryTables.OrderHeader.Ord_type, c_oOrderEntryTables.OrderHeader.OrderLines)

        EnableDisableFields(False)

    End Sub

    Private Sub EnableDisableFields(bEnable As Boolean)

        txtOrderNo.Enabled = bEnable
        cbOrderType.Enabled = bEnable
        txtCustomerNo.Enabled = bEnable

    End Sub

#End Region

#Region "Events"

    Private Sub btnCreateOrder_Click(sender As System.Object, e As System.EventArgs) Handles btnCreateOrder.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""
        Try
            '//Start Wisys SQL Transaction
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(True, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Create OE Order
            p_iWisysReturnValue = CreateOEOrder(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Create Order")
            End If

            '//Close Wisys Connection
            Dim p_eTransactionAction As Wisys.AllSystem.TrxEnums.TransactionAction
            p_eTransactionAction = If(p_iWisysReturnValue = SUCCESSFUL, Wisys.AllSystem.TrxEnums.TransactionAction.Commit, Wisys.AllSystem.TrxEnums.TransactionAction.Rollback)
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(p_eTransactionAction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If

            p_iWisysReturnValue = LoadOrder(cbOrderType.SelectedValue, txtOrderNo.Text, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Load Order")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Create Order")

        End Try

    End Sub

    Private Sub btnLoadOrder_Click(sender As System.Object, e As System.EventArgs) Handles btnLoadOrder.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""

        Try

            If (String.IsNullOrWhiteSpace(cbOrderType.Text)) Then
                MessageBox.Show("Enter An Order Type")
                Return
            End If

            If (String.IsNullOrWhiteSpace(txtOrderNo.Text)) Then
                MessageBox.Show("Enter An Order No")
                Return
            End If

            '//Open Wisys Connection 
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(False, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show("Open Connection: " & p_sWisysReturnMessage)
                Return
            End If

            p_iWisysReturnValue = LoadOrder(cbOrderType.SelectedValue, txtOrderNo.Text, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Load Order")
            End If

            '//Close Wisys Connection
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(TrxEnums.TransactionAction.NoTransaction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Load Order")
        End Try

    End Sub

    Private Sub btnUpdateOrder_Click(sender As System.Object, e As System.EventArgs) Handles btnUpdateOrder.Click

        Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""

        Try
            If (c_oOrderEntryTables Is Nothing OrElse _
                c_oOrderEntryTables.OrderHeader Is Nothing OrElse _
                String.IsNullOrWhiteSpace(c_oOrderEntryTables.OrderHeader.Ord_type) OrElse _
                String.IsNullOrWhiteSpace(c_oOrderEntryTables.OrderHeader.Ord_no)) Then

            End If

            '//Start Wisys SQL Transaction
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(True, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Update OE Order
            p_iWisysReturnValue = UpdateOEOrder(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Update Order")
            End If

            '//Close Wisys Connection
            Dim p_eTransactionAction As Wisys.AllSystem.TrxEnums.TransactionAction
            p_eTransactionAction = If(p_iWisysReturnValue = SUCCESSFUL, Wisys.AllSystem.TrxEnums.TransactionAction.Commit, Wisys.AllSystem.TrxEnums.TransactionAction.Rollback)
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(p_eTransactionAction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If

            p_iWisysReturnValue = LoadOrder(cbOrderType.SelectedValue, txtOrderNo.Text, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Load Order")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Update Order")

        End Try

    End Sub

    Private Sub btnClearScreen_Click(sender As System.Object, e As System.EventArgs) Handles btnClearScreen.Click

        InitializeUI()
        UcOELineGrid1.btnClearAll.PerformClick()

    End Sub

#End Region

End Class
