﻿Imports System.Data.SqlClient
Imports Wisys.AllSystem
Imports Wisys.ImTrx
Imports Wisys.Im

Public Class ucWBBinSerialLot

#Region "Variables"

    Private c_oWisysConnection As Wisys.AllSystem.ConnectionInfo

#End Region

#Region "Properties"

#End Region

#Region "Constructor"

    Public Sub New(oWisysConnection As Wisys.AllSystem.ConnectionInfo)

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        c_oWisysConnection = oWisysConnection

    End Sub

#End Region

#Region "Public Methods"

#End Region

#Region "Private Methods"

    Private Function ReceiptTransaction(ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = ReturnCodes.SUCCESSFUL
        Dim p_bIsBinned As Boolean = False
        Dim p_bIsSerialLot As Boolean = False

        '//Bin Collection
        Dim p_oWisysBinCollection As New Wisys.AllSystem.DataStructures.ItemBins
        Dim p_oWisysBin As Wisys.AllSystem.DataStructures.ItemBin = Nothing

        '//SerialLot Collection
        Dim p_oWisysSerialLotCollection As New Wisys.AllSystem.DataStructures.ItemSerialLots
        Dim p_oWisysSerialLot As Wisys.AllSystem.DataStructures.ItemSerialLot = Nothing

        Dim p_oReceipt As New Wisys.ImTrx.Receipt

        Try
            '//Get Bin Serial/Lot Information
            p_iWisysReturnValue = GetBinSerialLotInformation(txtItemNo.Text, txtLocation.Text, p_bIsBinned, p_bIsSerialLot, sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

            '//Set IM Receipt Conneciton
            p_oReceipt.Connection(c_oWisysConnection)

            '//Set Properties for IM Transaction
            p_oReceipt.ItemNumber = txtItemNo.Text
            p_oReceipt.Location = txtLocation.Text
            p_oReceipt.Quantity = CDbl(txtQuantity.Text)
            p_oReceipt.UserName = txtUserName.Text

            If (p_bIsBinned OrElse p_bIsSerialLot) Then
                If (ckUseFlatBinSerialLot.Checked) Then
                    '//Flat Entry is used if the entire transaction quantity will be applied to the same bin and/or serial/lot
                    p_oReceipt.BinNoFlatEntryValue = txtBinNo1.Text
                    p_oReceipt.SerialLotFlatEntryValue = txtSerialLotNo1_1.Text
                Else
                    '//Use Bin Serial/Lot collections if using multiple Bins or Serial/Lots across the transaction
                    If (p_bIsBinned) Then
                        If (p_bIsSerialLot) Then
                            p_oWisysBinCollection = BuildBinSerialLotCollection()
                            p_oReceipt.RecieptBins = p_oWisysBinCollection
                        Else
                            p_oWisysBinCollection = BuildBinCollection()
                            p_oReceipt.RecieptBins = p_oWisysBinCollection
                        End If
                    Else
                        p_oWisysSerialLotCollection = BuildSerialLotCollection()
                        p_oReceipt.SerialLots = p_oWisysSerialLotCollection
                    End If

                End If
            End If

            p_iWisysReturnValue = p_oReceipt.PostTrx(sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If

            txtTransactionID.Text = p_oReceipt.InvtrxRecordID

        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    Private Function GetBinSerialLotInformation(sItemNo As String, sLocation As String, ByRef bIsBinned As Boolean, ByRef bIsSeriaLotted As Boolean, ByRef sWisysReturnMessage As String) As Integer

        Dim p_iWisysReturnValue As Integer = ReturnCodes.SUCCESSFUL
        Dim p_oInventoryTables As New Wisys.Im.InventoryTables()
        Dim p_bRecordFound As Boolean = False

        Try
            bIsBinned = False
            bIsSeriaLotted = False

            p_oInventoryTables.Connection(c_oWisysConnection)
            p_iWisysReturnValue = p_oInventoryTables.Imitmidx.Read(sItemNo, sLocation, True, p_bRecordFound, sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                Return p_iWisysReturnValue
            End If
            If (p_bRecordFound = False) Then
                sWisysReturnMessage = Wisys.AllSystem.Constants.ITEM_LOC_NOT_ON_FILE
                Return ReturnCodes.FAILED
            End If

            bIsSeriaLotted = If(p_oInventoryTables.Imitmidx.Ser_lot_fg.Trim.ToUpper = "S" OrElse p_oInventoryTables.Imitmidx.Ser_lot_fg.Trim.ToUpper = "L", True, False)
            bIsBinned = p_oInventoryTables.Imitmidx.Iminvloc.Binned


        Catch ex As Exception
            sWisysReturnMessage = ex.Message
            p_iWisysReturnValue = ReturnCodes.FAILED

        End Try

        Return p_iWisysReturnValue

    End Function

    Private Function BuildBinCollection() As Wisys.AllSystem.DataStructures.ItemBins

        '//Bin Collection
        Dim p_oWisysBinCollection As New Wisys.AllSystem.DataStructures.ItemBins
        Dim p_oWisysBin As Wisys.AllSystem.DataStructures.ItemBin = Nothing
        Dim p_dBinQuantity As Double = 0

        '//In this hard coded example we'll assume we have a maximum of two Bins and use the First Quantity field for each Group
        '//***The Bin Quantity is always a positive number (i.e. a Negative Receipt would still have a postive bin quantity)
        If (String.IsNullOrWhiteSpace(txtBinNo1.Text) = False) Then
            p_dBinQuantity = 0
            Double.TryParse(txtQuantity1_1.Text, p_dBinQuantity)
            p_oWisysBin = p_oWisysBinCollection.add(txtBinNo1.Text, Math.Abs(p_dBinQuantity))
        End If

        If (String.IsNullOrWhiteSpace(txtBinNo2.Text) = False) Then
            p_dBinQuantity = 0
            Double.TryParse(txtQuantity2_1.Text, p_dBinQuantity)
            p_oWisysBin = p_oWisysBinCollection.add(txtBinNo2.Text, Math.Abs(p_dBinQuantity))
        End If

        Return p_oWisysBinCollection

    End Function

    Private Function BuildSerialLotCollection() As Wisys.AllSystem.DataStructures.ItemSerialLots

        '//SerialLot Collection
        Dim p_oWisysSerialLotCollection As New Wisys.AllSystem.DataStructures.ItemSerialLots
        Dim p_oWisysSerialLot As Wisys.AllSystem.DataStructures.ItemSerialLot = Nothing
        Dim p_dSerialLotQuantity As Double = 0

        '//In this hard coded example we'll assume we have a maximum of four Serial/Lots 
        '//***The Serial/Lot Quantity is always a positive number (i.e. a Negative Receipt would still have a postive Serial/Lot quantity)

        '//Add Serial/Lot 1
        If (String.IsNullOrWhiteSpace(txtSerialLotNo1_1.Text) = False) Then
            p_dSerialLotQuantity = 0
            Double.TryParse(txtQuantity1_1.Text, p_dSerialLotQuantity)
            p_oWisysSerialLot = p_oWisysSerialLotCollection.Add(txtQuantity1_1.Text, Math.Abs(p_dSerialLotQuantity))
            '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
            'p_oWisysSerialLot.EffectiveDate = DateTime.Now
            'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
            'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"
        End If

        '//Add Serial/Lot 2
        If (String.IsNullOrWhiteSpace(txtSerialLotNo1_2.Text) = False) Then
            p_dSerialLotQuantity = 0
            Double.TryParse(txtQuantity1_2.Text, p_dSerialLotQuantity)
            p_oWisysSerialLot = p_oWisysSerialLotCollection.Add(txtQuantity1_2.Text, Math.Abs(p_dSerialLotQuantity))
            '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
            'p_oWisysSerialLot.EffectiveDate = DateTime.Now
            'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
            'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"
        End If

        '//Add Serial/Lot 3
        If (String.IsNullOrWhiteSpace(txtSerialLotNo2_1.Text) = False) Then
            p_dSerialLotQuantity = 0
            Double.TryParse(txtQuantity2_1.Text, p_dSerialLotQuantity)
            p_oWisysSerialLot = p_oWisysSerialLotCollection.Add(txtQuantity2_1.Text, Math.Abs(p_dSerialLotQuantity))
            '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
            'p_oWisysSerialLot.EffectiveDate = DateTime.Now
            'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
            'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"
        End If

        '//Add Serial/Lot 4
        If (String.IsNullOrWhiteSpace(txtSerialLotNo2_2.Text) = False) Then
            p_dSerialLotQuantity = 0
            Double.TryParse(txtQuantity2_2.Text, p_dSerialLotQuantity)
            p_oWisysSerialLot = p_oWisysSerialLotCollection.Add(txtQuantity2_2.Text, Math.Abs(p_dSerialLotQuantity))
            '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
            'p_oWisysSerialLot.EffectiveDate = DateTime.Now
            'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
            'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"
        End If

        Return p_oWisysSerialLotCollection

    End Function

    Private Function BuildBinSerialLotCollection() As Wisys.AllSystem.DataStructures.ItemBins

        '//Bin Collection
        Dim p_oWisysBinCollection As New Wisys.AllSystem.DataStructures.ItemBins
        Dim p_oWisysBin As Wisys.AllSystem.DataStructures.ItemBin = Nothing

        '//SerialLot 
        Dim p_oWisysSerialLot As Wisys.AllSystem.DataStructures.ItemSerialLot = Nothing

        Dim p_dQuantity1 As Double = 0
        Dim p_dQuantity2 As Double = 0

        '//In this hard coded example we'll assume we have a maximum of two Bins with two Serial/Lots each  
        '//The sum of the Bin qauntities will match the receipt quantity
        '//The Bin quantity will be the sum of the associated two Serial/Lot quantities
        '//***The Bin and Serial/Lot Quantity is always a positive number (i.e. a Negative Receipt would still have a postive Bin and Serial/Lot quantity)

        '//Add Bin 1
        If (String.IsNullOrWhiteSpace(txtBinNo1.Text) = False) Then
            p_dQuantity1 = 0
            p_dQuantity2 = 0
            Double.TryParse(txtQuantity1_1.Text, p_dQuantity1)
            Double.TryParse(txtQuantity1_2.Text, p_dQuantity2)
            p_dQuantity1 = Math.Abs(p_dQuantity1)
            p_dQuantity2 = Math.Abs(p_dQuantity2)
            p_oWisysBin = p_oWisysBinCollection.add(txtBinNo1.Text, (p_dQuantity1 + p_dQuantity2))

            If (String.IsNullOrWhiteSpace(txtSerialLotNo1_1.Text) = False) Then
                p_oWisysSerialLot = p_oWisysBin.SerialLotNumbers.Add(txtSerialLotNo1_1.Text, p_dQuantity1)
                '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
                'p_oWisysSerialLot.EffectiveDate = DateTime.Now
                'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
                'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"

            End If

            If (String.IsNullOrWhiteSpace(txtSerialLotNo1_2.Text) = False) Then
                p_oWisysSerialLot = p_oWisysBin.SerialLotNumbers.Add(txtSerialLotNo1_2.Text, p_dQuantity2)

                '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
                'p_oWisysSerialLot.EffectiveDate = DateTime.Now
                'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
                'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"

            End If

        End If


        '//Add Bin 2
        If (String.IsNullOrWhiteSpace(txtBinNo2.Text) = False) Then
            p_dQuantity1 = 0
            p_dQuantity2 = 0
            Double.TryParse(txtQuantity2_1.Text, p_dQuantity1)
            Double.TryParse(txtQuantity2_2.Text, p_dQuantity2)
            p_dQuantity1 = Math.Abs(p_dQuantity1)
            p_dQuantity2 = Math.Abs(p_dQuantity2)
            p_oWisysBin = p_oWisysBinCollection.add(txtBinNo2.Text, (p_dQuantity1 + p_dQuantity2))

            If (String.IsNullOrWhiteSpace(txtSerialLotNo2_1.Text) = False) Then
                p_oWisysSerialLot = p_oWisysBin.SerialLotNumbers.Add(txtSerialLotNo2_1.Text, p_dQuantity1)
                '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
                'p_oWisysSerialLot.EffectiveDate = DateTime.Now
                'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
                'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"

            End If

            If (String.IsNullOrWhiteSpace(txtSerialLotNo2_2.Text) = False) Then
                p_oWisysSerialLot = p_oWisysBin.SerialLotNumbers.Add(txtSerialLotNo2_2.Text, p_dQuantity2)

                '//When adding a new Serial/Lot you have the option to modify underlying imlsmst fields
                'p_oWisysSerialLot.EffectiveDate = DateTime.Now
                'p_oWisysSerialLot.ExpirationDate = DateAdd(DateInterval.Year, 5, DateTime.Now)
                'p_oWisysSerialLot.MfgSerialLotNumber = "1234567890"

            End If

        End If

        Return p_oWisysBinCollection

    End Function

    Private Sub EnableDisableFlatBinSerialLot(bUseFlatEntry As Boolean)

        Dim p_bEnabled As Boolean = Not (bUseFlatEntry)

        txtQuantity1_1.Enabled = p_bEnabled
        txtSerialLotNo1_2.Enabled = p_bEnabled
        txtQuantity1_2.Enabled = p_bEnabled

        txtBinNo2.Enabled = p_bEnabled
        txtSerialLotNo2_1.Enabled = p_bEnabled
        txtQuantity2_1.Enabled = p_bEnabled
        txtSerialLotNo2_2.Enabled = p_bEnabled
        txtQuantity2_2.Enabled = p_bEnabled

        If (bUseFlatEntry) Then
            txtQuantity1_1.Text = ""
            txtSerialLotNo1_2.Text = ""
            txtQuantity1_2.Text = ""

            txtBinNo2.Text = ""
            txtSerialLotNo2_1.Text = ""
            txtQuantity2_1.Text = ""
            txtSerialLotNo2_2.Text = ""
            txtQuantity2_2.Text = ""
        End If

    End Sub

#End Region

#Region "Events"

    Private Sub btnPost_Click(sender As System.Object, e As System.EventArgs) Handles btnPost.Click

          Dim p_iWisysReturnValue As Integer = 0
        Dim p_sWisysReturnMessage As String = ""
        Try
            '//Start Wisys SQL Transaction
            p_iWisysReturnValue = c_oWisysConnection.OpenWisysConnection(True, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Open Connection")
                Return
            End If

            '//Receipt Transaction 
            p_iWisysReturnValue = ReceiptTransaction(p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Receipt Transaction")
            End If

            '//Close Wisys Connection
            Dim p_eTransactionAction As Wisys.AllSystem.TrxEnums.TransactionAction
            p_eTransactionAction = If(p_iWisysReturnValue = SUCCESSFUL, Wisys.AllSystem.TrxEnums.TransactionAction.Commit, Wisys.AllSystem.TrxEnums.TransactionAction.Rollback)
            p_iWisysReturnValue = c_oWisysConnection.CloseWisysConnection(p_eTransactionAction, p_sWisysReturnMessage)
            If (p_iWisysReturnValue <> ReturnCodes.SUCCESSFUL) Then
                MessageBox.Show(p_sWisysReturnMessage, "Close Connection")
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Receipt Transaction")

        End Try


    End Sub

    Private Sub ckUseFlatBinSerialLot_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckUseFlatBinSerialLot.CheckedChanged

        EnableDisableFlatBinSerialLot(ckUseFlatBinSerialLot.Checked)

    End Sub


#End Region

End Class
